package com.raycos.raycosdatacenterapi.scheduledtasks;


import com.raycos.raycoscommon.Internal.datacenterapi.Lock;
import com.raycos.raycoscommon.Internal.datacenterapi.TbAmzPullOrderTask;
import com.raycos.raycosdatacenterapi.amzorder.AmzPullOrderTaskService;
import com.raycos.raycosdatacenterapi.common.databaselock.LockService;
import com.raycos.raycosdatacenterapi.config.attach.SendAmzPullOrderMsg;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.Instant;
import java.util.List;

@Component
@Slf4j
public class SearchOrderPullTask {


    @Autowired
    private SendAmzPullOrderMsg sendAmzPullOrderMsg;
    @Autowired
    private LockService lockService;
    @Autowired
    private AmzPullOrderTaskService amzPullOrderTaskService;

    /**
     * 数据库锁key后缀
     */
    private final String LOCKKEYSUFFIX = "PULL_TASK";
    /**
     * 数据库锁 描述
     */
    private final String LOCKDESCRIBE = "亚马逊拉取订单任务_发送拉单任务到MQ中";


    /**
     * 运行中标识
     */
    private boolean RUNFLAG = false;

    // 1 分钟拉去一次
    @Scheduled(fixedRate = 60 * 1000)
    public void runTask() {
        try {
            if (!RUNFLAG) {
                sendOrderPullTask();
            }

        } catch (Exception e) {
            log.error("   异常", e);
        }

    }


    /**
     * 发送拉单任务到 MQ 队列中
     *
     * @throws Exception
     */
    public void sendOrderPullTask() throws Exception {

        // 查询 最后操作时间 小于当前时间 并且 重试次数 为零的 任务
        // 获取当前时间戳
        long second = Instant.now().getEpochSecond();
        TbAmzPullOrderTask task = new TbAmzPullOrderTask();
        task.setRetryCount((byte) 0);
        task.setLastOperationTime(second);
        List<TbAmzPullOrderTask> list = amzPullOrderTaskService.findPullTask(task);

        for (TbAmzPullOrderTask apo : list) {

            // 加锁 防止消息重复投递 lockKey =  companyId + storeId + "PULL_TASK"
            String lockKey = apo.getCompanyId() + apo.getStoreId() + LOCKKEYSUFFIX;
            Lock lock = new Lock();
            lock.setLockKey(lockKey);
            lock.setDescribe(LOCKDESCRIBE);

            if (!lockService.lock(lock)) {
                // 加锁失败证明 任务已经添加到拉单队列中了
                continue;
            }

            // 更行当前任务的重试次数为1
            this.updateRetryTimes(apo);

            // 发送 MQ
            sendAmzPullOrderMsg.sendMessage(apo);

        }


    }

    /**
     * 更新 亚马逊拉取订单任务  执行次数为1
     *
     * @param apo
     * @throws Exception
     */
    private void updateRetryTimes(TbAmzPullOrderTask apo) throws Exception {

        TbAmzPullOrderTask apot = new TbAmzPullOrderTask();
        apot.setId(apo.getId());
        apot.setRetryCount((byte) 1);
        amzPullOrderTaskService.editById(apot);
    }


}
